package nu.zoom.checked.server;

import java.io.IOException;
import java.net.StandardProtocolFamily;
import java.net.UnixDomainSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.ServerSocketChannel;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;

public class Main {
    public static void main(String[] args) throws IOException {
        new Main().run();
    }

    private void run() throws IOException {
        var path = Path.of("/tmp").resolve("udsserver.sock") ;
        Files.deleteIfExists(path);
        var socketAddress = UnixDomainSocketAddress.of(path);
        ServerSocketChannel serverChannel = ServerSocketChannel
                .open(StandardProtocolFamily.UNIX);
        serverChannel.bind(socketAddress);

        boolean keepRunning = true;
        while (keepRunning) {
            try (var clientChannel = serverChannel.accept()) {
                if (clientChannel != null) {
                    var buffer = ByteBuffer.allocate(500);
                    clientChannel.read(buffer);
                    buffer.flip();
                    var length = Byte.toUnsignedInt(buffer.get());
                    if (length < 400 && length > 0) {
                        var rawMessage = new byte[length];
                        buffer.get(rawMessage);
                        var message = new String(rawMessage, StandardCharsets.UTF_8);
                        System.out.println(message);
                        if (message.startsWith("q")) {
                            keepRunning = false;
                        }
                    }
                }
            }
        }

        serverChannel.close();
        Files.deleteIfExists(path);
    }
}